
#include <gtk/gtk.h>

#include <Rinternals.h>
#include <Rdefines.h>

#include "gtkUtils.h"

static GHashTable *R_clistCompareFuncs = NULL;

SEXP
R_gtk_set_clist_compare_func(SEXP s_clist, SEXP func)
{
    GtkCList *clist;

    clist = GTK_CLIST((GtkObject*) getPtrValue( s_clist ));
    if(!R_clistCompareFuncs)
	R_clistCompareFuncs = g_hash_table_new(NULL, NULL);

    R_PreserveObject(func);
    g_hash_table_insert(R_clistCompareFuncs, clist, func);

    return(R_NilValue);
}


SEXP
R_gtk_clist_set_compare_func(SEXP s_clist, SEXP routine)
{
    GtkCListCompareFunc fun;
    GtkCList *clist;

    clist = GTK_CLIST((GtkObject*) getPtrValue( s_clist ));
    fun = (GtkCListCompareFunc) R_ExternalPtrAddr(routine);
    gtk_clist_set_compare_func(clist, fun);

    return(R_NilValue);
}

gint
dummy_clist_compare_func(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{
    return 1;
}

gint
R_gtk_clist_compare_func_call(GtkCList *clist, gconstpointer ptr1, gconstpointer ptr2)
{

  gpointer p =  g_hash_table_lookup(R_clistCompareFuncs, (gpointer) clist);
  SEXP fun, e, ans;
  int status = 0;
  const GtkCListRow *row1 = (const GtkCListRow *) ptr1;
  const GtkCListRow *row2 = (const GtkCListRow *) ptr2;
  gchar *text1;
  gchar *text2;

  if(!p){
    PROBLEM "no compare function for the GtkCList"
    ERROR;
  } 


  text1 = GTK_CELL_TEXT (row1->cell[clist->sort_column])->text;
  text2 = GTK_CELL_TEXT (row2->cell[clist->sort_column])->text;
     
  fun = (SEXP) p;

  PROTECT(e = allocVector(LANGSXP, 3));

  SETCAR(e, fun);
  SETCAR(CDR(e), mkString(text1));
  SETCAR(CDR(CDR(e)), mkString(text2));

  ans = Rf_eval(e, R_GlobalEnv);  
  switch(TYPEOF(ans)) {
  case INTSXP:
    status = INTEGER(ans)[0];
    break;
  case LGLSXP:
    status = INTEGER(ans)[0];
    break;
  case REALSXP:
      status = REAL(ans)[0];
    break;
  default:
      PROBLEM "incorrect type returned from comparison function. Must be a number or logical"
      ERROR;
      break;

  }
  UNPROTECT(1);

  return(status);
}
